
#include <windows.h>
#include <stdio.h>
#include "connect.h"

#if defined (WIN32)
	#define IS_WIN32 TRUE
#else
	#define IS_WIN32 FALSE
#endif

#define IS_NT      IS_WIN32 && (BOOL)(GetVersion() < 0x80000000)
#define IS_WIN32S  IS_WIN32 && (BOOL)(!(IS_NT) && (LOBYTE(LOWORD(GetVersion()))<4))
#define IS_WIN95   (BOOL)(!(IS_NT) && !(IS_WIN32S)) && IS_WIN32


HINSTANCE hInst;   // current instance

LPCTSTR lpszAppName = "MyApp";
LPCTSTR lpszTitle   = "midiConnect()"; 


BOOL RegisterWin95( CONST WNDCLASS* lpwc );


int APIENTRY WinMain( HINSTANCE hInstance, HINSTANCE hPrevInstance,
                      LPTSTR lpCmdLine, int nCmdShow)
{
   MSG      msg;
   HWND     hWnd; 
   WNDCLASS wc;

   wc.style         = CS_HREDRAW | CS_VREDRAW | CS_OWNDC;
   wc.lpfnWndProc   = (WNDPROC)WndProc;       
   wc.cbClsExtra    = 0;                      
   wc.cbWndExtra    = 0;                      
   wc.hInstance     = hInstance;              
   wc.hIcon         = LoadIcon (hInstance, lpszAppName); 
   wc.hCursor       = LoadCursor(NULL, IDC_ARROW);
   wc.hbrBackground = (HBRUSH)(COLOR_WINDOW+1);
   wc.lpszMenuName  = lpszAppName;              
   wc.lpszClassName = lpszAppName;              

   if ( IS_WIN95 )
   {
      if ( !RegisterWin95( &wc ) )
         return( FALSE );
   }
   else if ( !RegisterClass( &wc ) )
      return( FALSE );

   hInst = hInstance; 

   hWnd = CreateWindow( lpszAppName, 
                        lpszTitle,    
                        WS_OVERLAPPEDWINDOW | WS_VSCROLL, 
                        CW_USEDEFAULT, 0, 
                        CW_USEDEFAULT, 0,  
                        NULL,              
                        NULL,              
                        hInstance,         
                        NULL               
                      );

   if ( !hWnd ) 
      return( FALSE );

   ShowWindow( hWnd, nCmdShow ); 
   UpdateWindow( hWnd );         

   while( GetMessage( &msg, NULL, 0, 0) )   
   {
      TranslateMessage( &msg ); 
      DispatchMessage( &msg );  
   }

   return( msg.wParam ); 
}


BOOL RegisterWin95( CONST WNDCLASS* lpwc )
{
	WNDCLASSEX wcex;

   wcex.style         = lpwc->style;
   wcex.lpfnWndProc   = lpwc->lpfnWndProc;
   wcex.cbClsExtra    = lpwc->cbClsExtra;
   wcex.cbWndExtra    = lpwc->cbWndExtra;
   wcex.hInstance     = lpwc->hInstance;
   wcex.hIcon         = lpwc->hIcon;
   wcex.hCursor       = lpwc->hCursor;
   wcex.hbrBackground = lpwc->hbrBackground;
   wcex.lpszMenuName  = lpwc->lpszMenuName;
   wcex.lpszClassName = lpwc->lpszClassName;

   // Added elements for Windows 95.
   //...............................
   wcex.cbSize = sizeof(WNDCLASSEX);
   wcex.hIconSm = LoadImage(wcex.hInstance, lpwc->lpszClassName, 
                            IMAGE_ICON, 16, 16,
                            LR_DEFAULTCOLOR );
			
   return RegisterClassEx( &wcex );
}

#define MSG_LEN          1024

// global variables
//.................

char     msg[MSG_LEN+1];

HWND     hListBox = NULL;
MMRESULT rc;

UINT     nDevId = 0;
HMIDIIN  hmi = NULL;
HMIDIOUT hmo = NULL;


LRESULT CALLBACK WndProc( HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam )
{
   switch( uMsg )
   {
      case WM_CREATE :
              // Create listbox
              //...............

              hListBox = CreateWindow( "LISTBOX", "",    
                                       WS_CHILD | LBS_NOTIFY | WS_VSCROLL | WS_BORDER | 
                                       WS_VISIBLE | LBS_NOINTEGRALHEIGHT, 
                                       0, 0, 
                                       0, 0,  
                                       hWnd,              
                                       (HMENU)101,              
                                       hInst,         
                                       NULL );
              break;

      case WM_SIZE :
              MoveWindow( hListBox, 0, 0, 
                          LOWORD( lParam ), 
                          HIWORD( lParam ), TRUE );
              break;

      case WM_COMMAND :
              switch( LOWORD( wParam ) )
              {
                 case IDM_TEST :
                        {
                           // open MIDI input device
                           //.......................

                           rc = midiInOpen(&hmi, nDevId, (DWORD)NULL, 
                                           (DWORD)NULL, CALLBACK_NULL);

                           if (rc != MMSYSERR_NOERROR)
                           {
                               midiInGetErrorText(rc, msg, MSG_LEN);
                               MessageBox(hWnd, msg, NULL, MB_OK);
                               break;
                           }

                           // open MIDI output device
                           //........................

                           rc = midiOutOpen(&hmo, nDevId, (DWORD)NULL, 
                                            (DWORD)NULL, CALLBACK_NULL);

                           if (rc != MMSYSERR_NOERROR)
                           {
                               midiInClose(hmi);
                               midiOutGetErrorText(rc, msg, MSG_LEN);
                               MessageBox(hWnd, msg, NULL, MB_OK);
                               break;
                           }
                           
                           // connect MIDI input device to 
                           // MIDI output device
                           //.............................

                           rc = midiConnect(hmi, hmo, NULL);

                           if (rc != MMSYSERR_NOERROR)
                           {
                               midiInClose(hmi);
                               midiOutClose(hmo);
                               midiOutGetErrorText(rc, msg, MSG_LEN);
                               MessageBox(hWnd, msg, NULL, MB_OK);
                               break;
                           }

                           // all data received from the MIDI input device
                           // will be sent to the MIDI output device
                           //.............................................

                           // do something here...
                           
                           // disconnect MIDI devices
                           //........................

                           midiDisconnect(hmi, hmo, NULL);

                           // close both MIDI devices
                           //........................

                           midiInClose(hmi);
                           midiOutClose(hmo);
                        }
                        break;

                 case IDM_ABOUT :
                        DialogBox( hInst, "AboutBox", hWnd, About );
                        break;

                 case IDM_EXIT :
                        DestroyWindow( hWnd );
                        break;
              }
              break;

      case WM_DESTROY :
              PostQuitMessage(0);
              break;

      default :
            return( DefWindowProc( hWnd, uMsg, wParam, lParam ) );
   }

   return( 0L );               
}


LRESULT CALLBACK About( HWND hDlg,           
                        UINT message,        
                        WPARAM wParam,       
                        LPARAM lParam)
{
   switch (message) 
   {
       case WM_INITDIALOG: 
               return (TRUE);

       case WM_COMMAND:                              
               if (   LOWORD(wParam) == IDOK         
                   || LOWORD(wParam) == IDCANCEL)    
               {
                       EndDialog(hDlg, TRUE);        
                       return (TRUE);
               }
               break;
   }

   return (FALSE); 
}




